@@ -51,6 +51,13 @@ namespace Google.Protobuf
             return new JsonReplayTokenizer(tokens, continuation);
         }
 
+        /// <summary>
+        /// The depth of recursion within JsonParser. This is not directly computable within the tokenizer itself,
+        /// as arrays contribute to the recursion depth for ListValue parsing, but not for "normal" message types.
+        /// This is maintained (and checked) by <see cref="JsonParser.Merge(IMessage, JsonTokenizer)"/>.
+        /// </summary>
+        internal int RecursionDepth { get; set; }
+
         /// <summary>
         /// Returns the depth of the stack, purely in objects (not collections).
         /// Informally, this is the number of remaining unclosed '{' characters we have.
