@@ -18,12 +18,13 @@ import java.util.Map;
 /** An adapter between the {@link Writer} interface and {@link CodedOutputStream}. */
 @CheckReturnValue
 @ExperimentalApi
+@SuppressWarnings({"unchecked", "rawtypes"})
 final class CodedOutputStreamWriter implements Writer {
   private final CodedOutputStream output;
 
   public static CodedOutputStreamWriter forCodedOutput(CodedOutputStream output) {
     if (output.wrapper != null) {
-      return output.wrapper;
+      return (CodedOutputStreamWriter) output.wrapper;
     }
     return new CodedOutputStreamWriter(output);
   }
@@ -142,9 +143,13 @@ final class CodedOutputStreamWriter implements Writer {
     output.writeGroup(fieldNumber, (MessageLite) value);
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   @Override
   public void writeGroup(int fieldNumber, Object value, Schema schema) throws IOException {
-    output.writeGroup(fieldNumber, (MessageLite) value, schema);
+    AbstractMessageLite<?, ?> message = (AbstractMessageLite) value;
+    output.writeTag(fieldNumber, WireFormat.WIRETYPE_START_GROUP);
+    schema.writeTo(message, this);
+    output.writeTag(fieldNumber, WireFormat.WIRETYPE_END_GROUP);
   }
 
   @Deprecated
@@ -334,6 +339,7 @@ final class CodedOutputStreamWriter implements Writer {
       }
     }
   }
+
   @Override
   public void writeUInt64List(int fieldNumber, List<Long> value, boolean packed)
       throws IOException {
