@@ -218,6 +218,7 @@ public abstract class GeneratedMessageLite<
     return (BuilderType2) dynamicMethod(MethodToInvoke.NEW_BUILDER, null, null);
   }
 
+  @SuppressWarnings("unchecked") // Guaranteed by runtime.
   protected final <
           MessageType2 extends GeneratedMessageLite<MessageType2, BuilderType2>,
           BuilderType2 extends GeneratedMessageLite.Builder<MessageType2, BuilderType2>>
@@ -226,6 +227,7 @@ public abstract class GeneratedMessageLite<
   }
 
   @Override
+  @SuppressWarnings("unchecked") // Guaranteed by runtime.
   public final boolean isInitialized() {
     return isInitialized((MessageType) this, /* shouldMemoize= */ true);
   }
@@ -310,6 +312,7 @@ public abstract class GeneratedMessageLite<
   }
 
   @Override
+  @SuppressWarnings("rawtypes")
   int getSerializedSize(
           Schema schema) {
     if (isMutable()) {
@@ -339,6 +342,7 @@ public abstract class GeneratedMessageLite<
     return getSerializedSize(null);
   }
 
+  @SuppressWarnings("unchecked")
   private int computeSerializedSize(
           Schema<?> nullableSchema) {
     if (nullableSchema == null) {
@@ -1139,7 +1143,7 @@ public abstract class GeneratedMessageLite<
           final Internal.EnumLiteMap<?> enumTypeMap,
           final int number,
           final WireFormat.FieldType type,
-          final Class singularType) {
+          final Class<?> singularType) {
     return new GeneratedExtension<ContainingType, Type>(
         containingTypeDefaultInstance,
         defaultValue,
@@ -1158,7 +1162,7 @@ public abstract class GeneratedMessageLite<
           final int number,
           final WireFormat.FieldType type,
           final boolean isPacked,
-          final Class singularType) {
+          final Class<?> singularType) {
     @SuppressWarnings("unchecked") // Subclasses ensure Type is a List
     Type emptyList = (Type) ProtobufArrayList.emptyList();
     return new GeneratedExtension<ContainingType, Type>(
@@ -1240,7 +1244,7 @@ public abstract class GeneratedMessageLite<
   // =================================================================
 
   /** Calls Class.getMethod and throws a RuntimeException if it fails. */
-  @SuppressWarnings("unchecked")
+  @SuppressWarnings({"unchecked", "rawtypes"})
   static Method getMethodOrDie(Class clazz, String name, Class... params) {
     try {
       return clazz.getMethod(name, params);
@@ -1292,7 +1296,7 @@ public abstract class GeneratedMessageLite<
         final Type defaultValue,
         final MessageLite messageDefaultInstance,
         final ExtensionDescriptor descriptor,
-        final Class singularType) {
+        final Class<?> singularType) {
       // Defensive checks to verify the correct initialization order of
       // GeneratedExtensions and their related GeneratedMessages.
       if (containingTypeDefaultInstance == null) {
