@@ -385,30 +385,34 @@
 }
 
 - (void)testThatItThrowsWhenWriteRawPtrFails {
-  NSOutputStream* output = [NSOutputStream outputStreamToMemory];
-  GPBCodedOutputStream* codedOutput =
-      [GPBCodedOutputStream streamWithOutputStream:output bufferSize:0];  // Skip buffering.
-  [output close];  // Close the output stream to force failure on write.
-  const char* cString = "raw";
+  const char* cString = "some test data to write";
+
+  // Make an outputString that won't be long enough for the data so the stream will
+  // eventually fail the write.
+  uint8_t buffer[2] = {0, 0};
+  NSOutputStream* output = [[[NSOutputStream alloc] initToBuffer:buffer
+                                                        capacity:sizeof(buffer)] autorelease];
+
+  // Use a tiny buffers so `writeRawPtr:offset:length:` will eventaully write directly.
+  GPBCodedOutputStream* codedOutput = [GPBCodedOutputStream streamWithOutputStream:output
+                                                                        bufferSize:1];
+
   XCTAssertThrowsSpecificNamed([codedOutput writeRawPtr:cString offset:0 length:strlen(cString)],
                                NSException, GPBCodedOutputStreamException_WriteFailed);
 }
 
 - (void)testThatDeallocNeverThrows {
+  // Output stream which can write precisely 1 byte of data before it is full.
   uint8_t buffer[1] = {0};
-
-  // Output stream which can write precisely 1 byte of data before it's full.
   NSOutputStream* output = [[[NSOutputStream alloc] initToBuffer:buffer
                                                         capacity:sizeof(buffer)] autorelease];
+  GPBCodedOutputStream* codedOutput = [[GPBCodedOutputStream alloc] initWithOutputStream:output];
 
-  NSMutableData* outputBuffer = [NSMutableData data];
-  GPBCodedOutputStream* codedOutput =
-      [[GPBCodedOutputStream alloc] initWithOutputStream:output data:outputBuffer];
-
+  // First byte and flush (filling it)
   [codedOutput writeRawByte:0x23];
   [codedOutput flush];
 
-  // Put one more byte in the output buffer.
+  // One more byte will fail on flush due to lack of space in the stream.
   [codedOutput writeRawByte:0x42];
   XCTAssertThrowsSpecificNamed([codedOutput flush], NSException,
                                GPBCodedOutputStreamException_WriteFailed);
