@@ -15,16 +15,6 @@
 
 upb_DefPool* get_global_symtab();
 
-// In PHP 8.1, mismatched tentative return types emit a deprecation notice.
-// https://wiki.php.net/rfc/internal_method_return_types
-//
-// When compiling for earlier php versions, the return type is dropped.
-#if PHP_VERSION_ID < 80100
-#define ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(       \
-    name, return_reference, required_num_args, type, allow_null) \
-  ZEND_BEGIN_ARG_INFO_EX(name, return_reference, required_num_args, allow_null)
-#endif
-
 ZEND_BEGIN_ARG_INFO(arginfo_void, 0)
 ZEND_END_ARG_INFO()
 
@@ -32,7 +22,7 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_setter, 0, 0, 1)
 ZEND_ARG_INFO(0, value)
 ZEND_END_ARG_INFO()
 
-#define PHP_PROTOBUF_VERSION "4.33.4"
+#define PHP_PROTOBUF_VERSION "5.34.0"
 
 // ptr -> PHP object cache. This is a weak map that caches lazily-created
 // wrapper objects around upb types:
