@@ -13,11 +13,15 @@
 #include <Python.h>
 #include <frameobject.h>
 
+#include <cstddef>
 #include <cstdint>
+#include <cstring>
 #include <string>
-#include <unordered_map>
 
 #include "google/protobuf/descriptor.pb.h"
+#include "absl/base/attributes.h"
+#include "absl/base/const_init.h"
+#include "absl/base/thread_annotations.h"
 #include "absl/container/flat_hash_map.h"
 #include "absl/log/absl_check.h"
 #include "absl/strings/string_view.h"
@@ -27,6 +31,7 @@
 #include "google/protobuf/io/coded_stream.h"
 #include "google/protobuf/pyext/descriptor_containers.h"
 #include "google/protobuf/pyext/descriptor_pool.h"
+#include "google/protobuf/pyext/free_threading_mutex.h"
 #include "google/protobuf/pyext/message.h"
 #include "google/protobuf/pyext/message_factory.h"
 #include "google/protobuf/pyext/scoped_pyobject_ptr.h"
@@ -70,17 +75,23 @@ namespace google {
 namespace protobuf {
 namespace python {
 
+// Mutex to protect interned_descriptors from concurrent access in
+// free-threading Python builds. Zero-cost in GIL-enabled builds.
+// NOTE: Free-threading support is still experimental.
+FreeThreadingMutex interned_descriptors_mutex(absl::kConstInit);
+
 // Store interned descriptors, so that the same C++ descriptor yields the same
 // Python object. Objects are not immortal: this map does not own the
 // references, and items are deleted when the last reference to the object is
 // released.
 // This is enough to support the "is" operator on live objects.
 // All descriptors are stored here.
-std::unordered_map<const void*, PyObject*>* interned_descriptors;
+absl::flat_hash_map<const void*, PyObject*>* interned_descriptors
+    ABSL_PT_GUARDED_BY(interned_descriptors_mutex);
 
 PyObject* PyString_FromCppString(absl::string_view str) {
   return PyUnicode_FromStringAndSize(str.data(),
-                                     static_cast<size_t>(str.size()));
+                                     static_cast<Py_ssize_t>(str.size()));
 }
 
 // Check that the calling Python code is the global scope of a _pb2.py module.
@@ -217,7 +228,8 @@ bool Reparse(PyMessageFactory* message_factory, const Message& from,
              Message* to) {
   // Reparse message.
   std::string serialized;
-  from.SerializeToString(&serialized);
+  // TODO: Remove this suppression.
+  (void)from.SerializeToString(&serialized);
   io::CodedInputStream input(
       reinterpret_cast<const uint8_t*>(serialized.c_str()), serialized.size());
   input.SetExtensionRegistry(message_factory->pool->pool,
@@ -399,13 +411,21 @@ PyObject* NewInternedDescriptor(PyTypeObject* type,
   }
 
   // See if the object is in the map of interned descriptors
-  std::unordered_map<const void*, PyObject*>::iterator it =
-      interned_descriptors->find(descriptor);
-  if (it != interned_descriptors->end()) {
-    ABSL_DCHECK(Py_TYPE(it->second) == type);
-    Py_INCREF(it->second);
-    return it->second;
+  PyObject* existing = nullptr;
+  {
+    FreeThreadingLockGuard lock(interned_descriptors_mutex);
+    auto it = interned_descriptors->find(descriptor);
+    if (it != interned_descriptors->end()) {
+      ABSL_DCHECK(Py_TYPE(it->second) == type);
+      existing = it->second;
+    }
   }
+  // Py_INCREF must be called outside the lock to avoid deadlock
+  if (existing != nullptr) {
+    Py_INCREF(existing);
+    return existing;
+  }
+
   // Create a new descriptor object
   PyBaseDescriptor* py_descriptor = PyObject_GC_New(PyBaseDescriptor, type);
   if (py_descriptor == nullptr) {
@@ -413,10 +433,6 @@ PyObject* NewInternedDescriptor(PyTypeObject* type,
   }
   py_descriptor->descriptor = descriptor;
 
-  // and cache it.
-  interned_descriptors->insert(
-      std::make_pair(descriptor, reinterpret_cast<PyObject*>(py_descriptor)));
-
   // Ensures that the DescriptorPool stays alive.
   PyDescriptorPool* pool =
       GetDescriptorPool_FromPool(GetFileDescriptor(descriptor)->pool());
@@ -430,6 +446,26 @@ PyObject* NewInternedDescriptor(PyTypeObject* type,
 
   PyObject_GC_Track(py_descriptor);
 
+  // Cache the fully initialized descriptor.
+  // Check again if another thread cached it while we were initializing.
+  {
+    FreeThreadingLockGuard lock(interned_descriptors_mutex);
+    auto [it, inserted] = interned_descriptors->insert(
+        std::make_pair(descriptor, reinterpret_cast<PyObject*>(py_descriptor)));
+    if (!inserted) {
+      // Another thread beat us to it. Use the existing descriptor.
+      ABSL_DCHECK(Py_TYPE(it->second) == type);
+      existing = it->second;
+    }
+  }
+
+  // If another thread cached first, clean up our descriptor and use theirs
+  if (existing != nullptr) {
+    Py_DECREF(py_descriptor);
+    Py_INCREF(existing);
+    return existing;
+  }
+
   if (was_created) {
     *was_created = true;
   }
@@ -439,8 +475,12 @@ PyObject* NewInternedDescriptor(PyTypeObject* type,
 static void Dealloc(PyObject* pself) {
   PyBaseDescriptor* self = reinterpret_cast<PyBaseDescriptor*>(pself);
   // Remove from interned dictionary
-  interned_descriptors->erase(self->descriptor);
+  {
+    FreeThreadingLockGuard lock(interned_descriptors_mutex);
+    interned_descriptors->erase(self->descriptor);
+  }
   Py_CLEAR(self->pool);
+  PyObject_GC_UnTrack(pself);
   Py_TYPE(self)->tp_free(pself);
 }
 
@@ -824,22 +864,6 @@ static PyObject* GetCppType(PyBaseDescriptor* self, void* closure) {
   return PyLong_FromLong(_GetDescriptor(self)->cpp_type());
 }
 
-static void WarnDeprecatedLabel() {
-  static int deprecated_label_count = 100;
-  if (deprecated_label_count > 0) {
-    --deprecated_label_count;
-    PyErr_WarnEx(
-        PyExc_DeprecationWarning,
-        "label() is deprecated. Use is_required() or is_repeated() instead.",
-        3);
-  }
-}
-
-static PyObject* GetLabel(PyBaseDescriptor* self, void* closure) {
-  WarnDeprecatedLabel();
-  return PyLong_FromLong(_GetDescriptor(self)->label());
-}
-
 static PyObject* IsRequired(PyBaseDescriptor* self, void* closure) {
   return PyBool_FromLong(_GetDescriptor(self)->is_required());
 }
@@ -1055,7 +1079,6 @@ static PyGetSetDef Getters[] = {
     {"file", (getter)GetFile, nullptr, "File Descriptor"},
     {"type", (getter)GetType, nullptr, "C++ Type"},
     {"cpp_type", (getter)GetCppType, nullptr, "C++ Type"},
-    {"label", (getter)GetLabel, nullptr, "Label"},
     {"is_required", (getter)IsRequired, nullptr, "Is Required"},
     {"is_repeated", (getter)IsRepeated, nullptr, "Is Repeated"},
     {"number", (getter)GetNumber, nullptr, "Number"},
@@ -1468,7 +1491,8 @@ static PyObject* GetSerializedPb(PyFileDescriptor* self, void* closure) {
   FileDescriptorProto file_proto;
   _GetDescriptor(self)->CopyTo(&file_proto);
   std::string contents;
-  file_proto.SerializePartialToString(&contents);
+  // TODO: Remove this suppression.
+  (void)file_proto.SerializePartialToString(&contents);
   self->serialized_pb = PyBytes_FromStringAndSize(
       contents.c_str(), static_cast<size_t>(contents.size()));
   if (self->serialized_pb == nullptr) {
@@ -2128,7 +2152,7 @@ bool InitDescriptor() {
   if (!InitDescriptorMappingTypes()) return false;
 
   // Initialize globals defined in this file.
-  interned_descriptors = new std::unordered_map<const void*, PyObject*>;
+  interned_descriptors = new absl::flat_hash_map<const void*, PyObject*>;
 
   return true;
 }
