@@ -7,8 +7,8 @@
 
 //! Traits that are implemented by codegen types.
 
+use crate::MutProxied;
 use crate::__internal::SealedInternal;
-use crate::{MutProxied, MutProxy, ViewProxy};
 use create::Parse;
 use interop::{MessageMutInterop, MessageViewInterop, OwnedMessageInterop};
 use read::Serialize;
@@ -35,7 +35,6 @@ pub trait Message: SealedInternal
 
 /// A trait that all generated message views implement.
 pub trait MessageView<'msg>: SealedInternal
-    + ViewProxy<'msg, Proxied = Self::Message>
     // Read traits:
     + Debug + Serialize + Default
     // Thread safety:
@@ -51,13 +50,12 @@ pub trait MessageView<'msg>: SealedInternal
 
 /// A trait that all generated message muts implement.
 pub trait MessageMut<'msg>: SealedInternal
-    + MutProxy<'msg, MutProxied = Self::Message>
     // Read traits:
     + Debug + Serialize
     // Write traits:
     + Clear + ClearAndParse + TakeFrom + CopyFrom + MergeFrom
     // Thread safety:
-    + Sync
+    + Send + Sync
     // Copy/Clone:
     // (Neither)
     // C++ Interop:
@@ -275,4 +273,14 @@ pub(crate) mod interop {
         #[cfg(cpp_kernel)]
         unsafe fn __unstable_wrap_raw_message_mut_unchecked_lifetime(raw: *mut c_void) -> Self;
     }
+
+    /// Trait related to message descriptors.
+    /// Note that this is only implemented for the types implementing
+    /// `proto2::Message`.
+    #[cfg(all(cpp_kernel, not(lite_runtime)))]
+    pub trait MessageDescriptorInterop {
+        /// Returns a pointer to a `proto2::Descriptor` or `nullptr` if the
+        /// descriptor is not available.
+        fn __unstable_get_descriptor() -> *const std::ffi::c_void;
+    }
 }
