@@ -5,7 +5,7 @@
 // license that can be found in the LICENSE file or at
 // https://developers.google.com/open-source/licenses/bsd
 use crate::__internal::SealedInternal;
-use crate::{AsView, IntoView, Proxied, Proxy, ViewProxy};
+use crate::{AsView, IntoView, Proxied};
 
 macro_rules! impl_singular_primitives {
   ($($t:ty),*) => {
@@ -16,9 +16,6 @@ macro_rules! impl_singular_primitives {
             type View<'msg> = $t;
         }
 
-        impl<'msg> Proxy<'msg> for $t {
-        }
-
         impl AsView for $t {
             type Proxied = $t;
 
@@ -35,8 +32,6 @@ macro_rules! impl_singular_primitives {
           }
         }
 
-        impl<'msg> ViewProxy<'msg> for $t {}
-
         // ProxiedInRepeated is implemented in {cpp,upb}.rs
       )*
   }
