@@ -44,33 +44,32 @@
 //! implemented the concept of "proxy" types. Proxy types are a reference-like
 //! indirection between the user and the internal memory representation.
 
-use crate::RepeatedMut;
-use crate::__internal::Private;
-use crate::repeated::ProxiedInRepeated;
-use std::fmt::Debug;
-use std::marker::{Send, Sync};
+use crate::__internal::{Private, SealedInternal};
 
 /// A type that can be accessed through a reference-like proxy.
 ///
-/// An instance of a `Proxied` can be accessed
-/// immutably via `Proxied::View` and mutably via `Proxied::Mut`.
+/// An instance of a `Proxied` can be accessed immutably via `Proxied::View`.
 ///
 /// All Protobuf field types implement `Proxied`.
-pub trait Proxied {
+pub trait Proxied: SealedInternal + AsView<Proxied = Self> + Sized + 'static {
     /// The proxy type that provides shared access to a `T`, like a `&'msg T`.
     ///
     /// Most code should use the type alias [`View`].
-    type View<'msg>: ViewProxy<'msg, Proxied = Self> + Copy + Send + SettableValue<Self>
-    where
-        Self: 'msg;
+    type View<'msg>: AsView<Proxied = Self> + IntoView<'msg>;
+}
 
+/// A type that can be be accessed through a reference-like proxy.
+///
+/// An instance of a `MutProxied` can be accessed mutably via `MutProxied::Mut`
+/// and immutably via `MutProxied::View`.
+///
+/// `MutProxied` is implemented by message, map and repeated field types.
+pub trait MutProxied: SealedInternal + Proxied + AsMut<MutProxied = Self> + 'static {
     /// The proxy type that provides exclusive mutable access to a `T`, like a
     /// `&'msg mut T`.
     ///
     /// Most code should use the type alias [`Mut`].
-    type Mut<'msg>: MutProxy<'msg, Proxied = Self>
-    where
-        Self: 'msg;
+    type Mut<'msg>: AsMut<MutProxied = Self> + IntoMut<'msg> + IntoView<'msg>;
 }
 
 /// A proxy type that provides shared access to a `T`, like a `&'msg T`.
@@ -84,14 +83,15 @@ pub type View<'msg, T> = <T as Proxied>::View<'msg>;
 ///
 /// This is more concise than fully spelling the associated type.
 #[allow(dead_code)]
-pub type Mut<'msg, T> = <T as Proxied>::Mut<'msg>;
+pub type Mut<'msg, T> = <T as MutProxied>::Mut<'msg>;
 
-/// Declares conversion operations common to all views.
+/// Used to semantically do a cheap "to-reference" conversion. This is
+/// implemented on both owned `Proxied` types as well as view and mut proxy
+/// types.
 ///
-/// This trait is intentionally made non-object-safe to prevent a potential
-/// future incompatible change.
-pub trait ViewProxy<'msg>: 'msg + Sync + Unpin + Sized + Debug {
-    type Proxied: 'msg + Proxied + ?Sized;
+/// On a view proxy this will behave as a reborrow into a shorter lifetime.
+pub trait AsView: SealedInternal {
+    type Proxied: Proxied;
 
     /// Converts a borrow into a `View` with the lifetime of that borrow.
     ///
@@ -103,7 +103,7 @@ pub trait ViewProxy<'msg>: 'msg + Sync + Unpin + Sized + Debug {
     ///
     /// For example, the call to `.as_view()` in the following snippet
     /// wouldn't be necessary in concrete code:
-    /// ```
+    /// ```ignore
     /// fn reborrow<'a, 'b, T>(x: &'b View<'a, T>) -> View<'b, T>
     /// where 'a: 'b, T: Proxied
     /// {
@@ -113,7 +113,16 @@ pub trait ViewProxy<'msg>: 'msg + Sync + Unpin + Sized + Debug {
     ///
     /// [invariant]: https://doc.rust-lang.org/nomicon/subtyping.html#variance
     fn as_view(&self) -> View<'_, Self::Proxied>;
+}
 
+/// Used to turn another 'borrow' into a view proxy.
+///
+/// On a mut proxy this borrows to a View (semantically matching turning a `&mut
+/// T` into a `&T`).
+///
+/// On a view proxy this will behave as a reborrow into a shorter lifetime
+/// (semantically matching a `&'a T` into a `&'b T` where `'a: 'b`).
+pub trait IntoView<'msg>: SealedInternal + AsView {
     /// Converts into a `View` with a potentially shorter lifetime.
     ///
     /// In non-generic code we don't need to use `into_view` because the proxy
@@ -122,13 +131,13 @@ pub trait ViewProxy<'msg>: 'msg + Sync + Unpin + Sized + Debug {
     /// `into_view` to explicitly perform the operation that in concrete
     /// code coercion would perform implicitly.
     ///
-    /// ```
+    /// ```ignore
     /// fn reborrow_generic_view_into_view<'a, 'b, T>(
     ///     x: View<'a, T>,
     ///     y: View<'b, T>,
     /// ) -> [View<'b, T>; 2]
     /// where
-    ///     T: Proxied,
+    ///     T: MutProxied,
     ///     'a: 'b,
     /// {
     ///     // `[x, y]` fails to compile because `'a` is not the same as `'b` and the `View`
@@ -144,42 +153,22 @@ pub trait ViewProxy<'msg>: 'msg + Sync + Unpin + Sized + Debug {
         'msg: 'shorter;
 }
 
-/// Declares operations common to all mutators.
+/// Used to semantically do a cheap "to-mut-reference" conversion. This is
+/// implemented on both owned `Proxied` types as well as mut proxy types.
 ///
-/// This trait is intentionally made non-object-safe to prevent a potential
-/// future incompatible change.
-pub trait MutProxy<'msg>: ViewProxy<'msg> {
-    /// Gets an immutable view of this field. This is shorthand for `as_view`.
-    ///
-    /// This provides a shorter lifetime than `into_view` but can also be called
-    /// multiple times - if the result of `get` is not living long enough
-    /// for your use, use that instead.
-    fn get(&self) -> View<'_, Self::Proxied> {
-        self.as_view()
-    }
-
-    /// Sets this field to the given `val`.
-    ///
-    /// Any borrowed data from `val` will be cloned.
-    fn set(&mut self, val: impl SettableValue<Self::Proxied>) {
-        val.set_on(Private, self.as_mut())
-    }
+/// On a mut proxy this will behave as a reborrow into a shorter lifetime.
+pub trait AsMut: SealedInternal + AsView<Proxied = Self::MutProxied> {
+    type MutProxied: MutProxied;
 
     /// Converts a borrow into a `Mut` with the lifetime of that borrow.
-    ///
-    /// This function enables calling multiple methods consuming `self`, for
-    /// example:
-    ///
-    /// ```ignore
-    ///   let mut sub: Mut<SubMsg> = msg.submsg_mut().or_default();
-    ///   sub.as_mut().field_x_mut().set(10);  // field_x_mut is fn(self)
-    ///   sub.field_y_mut().set(20);  // `sub` is now consumed
-    /// ```
-    ///
-    /// `as_mut` is also useful in generic code to explicitly perform the
-    /// operation that in concrete code coercion would perform implicitly.
-    fn as_mut(&mut self) -> Mut<'_, Self::Proxied>;
+    fn as_mut(&mut self) -> Mut<'_, Self::MutProxied>;
+}
 
+/// Used to turn another 'borrow' into a mut proxy.
+///
+/// On a mut proxy this will behave as a reborrow into a shorter lifetime
+/// (semantically matching a `&mut 'a T` into a `&mut 'b T` where `'a: 'b`).
+pub trait IntoMut<'msg>: SealedInternal + AsMut {
     /// Converts into a `Mut` with a potentially shorter lifetime.
     ///
     /// In non-generic code we don't need to use `into_mut` because the proxy
@@ -188,7 +177,7 @@ pub trait MutProxy<'msg>: ViewProxy<'msg> {
     /// `into_mut` to explicitly perform the operation that in concrete code
     /// coercion would perform implicitly.
     ///
-    /// ```
+    /// ```ignore
     /// fn reborrow_generic_mut_into_mut<'a, 'b, T>(x: Mut<'a, T>, y: Mut<'b, T>) -> [Mut<'b, T>; 2]
     /// where
     ///     T: Proxied,
@@ -202,87 +191,28 @@ pub trait MutProxy<'msg>: ViewProxy<'msg> {
     /// ```
     ///
     /// [invariant]: https://doc.rust-lang.org/nomicon/subtyping.html#variance
-    fn into_mut<'shorter>(self) -> Mut<'shorter, Self::Proxied>
+    fn into_mut<'shorter>(self) -> Mut<'shorter, Self::MutProxied>
     where
         'msg: 'shorter;
 }
 
-// TODO: move this to `optional.rs` as it's only used for optionals
-/// `Proxied` types that can be optionally set or unset.
+/// A value to `Proxied`-value conversion that consumes the input value.
 ///
-/// All scalar and message types implement `ProxiedWithPresence`, while repeated
-/// types don't.
-pub trait ProxiedWithPresence: Proxied {
-    /// The data necessary to store a present field mutator proxying `Self`.
-    /// This is the contents of `PresentField<'msg, Self>`.
-    type PresentMutData<'msg>: MutProxy<'msg, Proxied = Self>;
-
-    /// The data necessary to store an absent field mutator proxying `Self`.
-    /// This is the contents of `AbsentField<'msg, Self>`.
-    type AbsentMutData<'msg>: ViewProxy<'msg, Proxied = Self>;
-
-    /// Clears a present field.
-    fn clear_present_field(present_mutator: Self::PresentMutData<'_>) -> Self::AbsentMutData<'_>;
-
-    /// Sets an absent field to its default value.
-    ///
-    /// This can be more efficient than setting with a default value, e.g.
-    /// a default submessage could share resources with the parent message.
-    fn set_absent_to_default(absent_mutator: Self::AbsentMutData<'_>) -> Self::PresentMutData<'_>;
+/// All setter functions accept types that implement `IntoProxied`. The purpose
+/// of `IntoProxied` is to allow setting arbitrary values on Protobuf fields
+/// with the minimal number of copies.
+///
+/// This trait must not be implemented on types outside the Protobuf codegen and
+/// runtime. We expect it to change in backwards incompatible ways in the
+/// future.
+pub trait IntoProxied<T: Proxied> {
+    #[doc(hidden)]
+    fn into_proxied(self, _private: Private) -> T;
 }
 
-/// Values that can be used to set a field of `T`.
-pub trait SettableValue<T>: Sized
-where
-    T: Proxied + ?Sized,
-{
-    /// Consumes `self` to set the given mutator to the value of `self`.
-    #[doc(hidden)]
-    fn set_on<'msg>(self, _private: Private, mutator: Mut<'msg, T>)
-    where
-        T: 'msg;
-
-    /// Consumes `self` and `absent_mutator` to set the given empty field to
-    /// the value of `self`.
-    #[doc(hidden)]
-    fn set_on_absent(
-        self,
-        _private: Private,
-        absent_mutator: T::AbsentMutData<'_>,
-    ) -> T::PresentMutData<'_>
-    where
-        T: ProxiedWithPresence,
-    {
-        let mut present = T::set_absent_to_default(absent_mutator);
-        self.set_on(Private, present.as_mut());
-        present
-    }
-
-    /// Consumes `self` and `present_mutator` to set the given present field
-    /// to the value of `self`.
-    #[doc(hidden)]
-    fn set_on_present(self, _private: Private, mut present_mutator: T::PresentMutData<'_>)
-    where
-        T: ProxiedWithPresence,
-    {
-        self.set_on(Private, present_mutator.as_mut())
-    }
-
-    /// Consumes `self` and `repeated_mutator` to set the value at the
-    /// given index to the value of `self`.
-    ///
-    /// # Safety
-    /// `index` must be less than `repeated_mutator.len()`
-    #[doc(hidden)]
-    unsafe fn set_on_repeated_unchecked(
-        self,
-        _private: Private,
-        mut _repeated_mutator: RepeatedMut<T>,
-        _index: usize,
-    ) where
-        T: ProxiedInRepeated,
-    {
-        unimplemented!()
+impl<T: Proxied> IntoProxied<T> for T {
+    fn into_proxied(self, _private: Private) -> T {
+        self
     }
 }
 
@@ -290,7 +220,6 @@ where
 mod tests {
     use super::*;
     use googletest::prelude::*;
-    use std::borrow::Cow;
 
     #[derive(Debug, Default, PartialEq)]
     struct MyProxied {
@@ -307,30 +236,53 @@ mod tests {
         }
     }
 
+    impl SealedInternal for MyProxied {}
+
     impl Proxied for MyProxied {
         type View<'msg> = MyProxiedView<'msg>;
+    }
+
+    impl AsView for MyProxied {
+        type Proxied = Self;
+        fn as_view(&self) -> MyProxiedView<'_> {
+            self.as_view()
+        }
+    }
+
+    impl MutProxied for MyProxied {
         type Mut<'msg> = MyProxiedMut<'msg>;
     }
 
+    impl AsMut for MyProxied {
+        type MutProxied = Self;
+        fn as_mut(&mut self) -> MyProxiedMut<'_> {
+            self.as_mut()
+        }
+    }
+
     #[derive(Debug, Clone, Copy)]
     struct MyProxiedView<'msg> {
         my_proxied_ref: &'msg MyProxied,
     }
 
+    impl<'msg> SealedInternal for MyProxiedView<'msg> {}
+
     impl MyProxiedView<'_> {
         fn val(&self) -> &str {
             &self.my_proxied_ref.val
         }
     }
 
-    impl<'msg> ViewProxy<'msg> for MyProxiedView<'msg> {
+    impl<'msg> AsView for MyProxiedView<'msg> {
         type Proxied = MyProxied;
 
-        fn as_view(&self) -> View<'msg, MyProxied> {
+        fn as_view(&self) -> MyProxiedView<'msg> {
             *self
         }
+    }
 
-        fn into_view<'shorter>(self) -> View<'shorter, MyProxied>
+    impl<'msg> IntoView<'msg> for MyProxiedView<'msg> {
+        fn into_view<'shorter>(self) -> MyProxiedView<'shorter>
         where
             'msg: 'shorter,
         {
@@ -343,12 +295,17 @@ mod tests {
         my_proxied_ref: &'msg mut MyProxied,
     }
 
-    impl<'msg> ViewProxy<'msg> for MyProxiedMut<'msg> {
+    impl<'msg> SealedInternal for MyProxiedMut<'msg> {}
+
+    impl<'msg> AsView for MyProxiedMut<'msg> {
         type Proxied = MyProxied;
 
-        fn as_view(&self) -> View<'_, MyProxied> {
+        fn as_view(&self) -> MyProxiedView<'_> {
             MyProxiedView { my_proxied_ref: self.my_proxied_ref }
         }
+    }
+
+    impl<'msg> IntoView<'msg> for MyProxiedMut<'msg> {
         fn into_view<'shorter>(self) -> View<'shorter, MyProxied>
         where
             'msg: 'shorter,
@@ -357,12 +314,16 @@ mod tests {
         }
     }
 
-    impl<'msg> MutProxy<'msg> for MyProxiedMut<'msg> {
-        fn as_mut(&mut self) -> Mut<'_, MyProxied> {
+    impl<'msg> AsMut for MyProxiedMut<'msg> {
+        type MutProxied = MyProxied;
+
+        fn as_mut(&mut self) -> MyProxiedMut<'_> {
             MyProxiedMut { my_proxied_ref: self.my_proxied_ref }
         }
+    }
 
-        fn into_mut<'shorter>(self) -> Mut<'shorter, MyProxied>
+    impl<'msg> IntoMut<'msg> for MyProxiedMut<'msg> {
+        fn into_mut<'shorter>(self) -> MyProxiedMut<'shorter>
         where
             'msg: 'shorter,
         {
@@ -370,46 +331,7 @@ mod tests {
         }
     }
 
-    impl SettableValue<MyProxied> for MyProxiedView<'_> {
-        fn set_on<'msg>(self, _private: Private, mutator: Mut<'msg, MyProxied>)
-        where
-            MyProxied: 'msg,
-        {
-            mutator.my_proxied_ref.val = self.my_proxied_ref.val.clone();
-        }
-    }
-
-    impl SettableValue<MyProxied> for String {
-        fn set_on<'msg>(self, _private: Private, mutator: Mut<'msg, MyProxied>)
-        where
-            MyProxied: 'msg,
-        {
-            mutator.my_proxied_ref.val = self;
-        }
-    }
-
-    impl SettableValue<MyProxied> for &'_ str {
-        fn set_on<'msg>(self, _private: Private, mutator: Mut<'msg, MyProxied>)
-        where
-            MyProxied: 'msg,
-        {
-            mutator.my_proxied_ref.val.replace_range(.., self);
-        }
-    }
-
-    impl SettableValue<MyProxied> for Cow<'_, str> {
-        fn set_on<'msg>(self, _private: Private, mutator: Mut<'msg, MyProxied>)
-        where
-            MyProxied: 'msg,
-        {
-            match self {
-                Cow::Owned(x) => <String as SettableValue<MyProxied>>::set_on(x, Private, mutator),
-                Cow::Borrowed(x) => <&str as SettableValue<MyProxied>>::set_on(x, Private, mutator),
-            }
-        }
-    }
-
-    #[test]
+    #[gtest]
     fn test_as_view() {
         let my_proxied = MyProxied { val: "Hello World".to_string() };
 
@@ -418,25 +340,13 @@ mod tests {
         assert_that!(my_view.val(), eq(&my_proxied.val));
     }
 
-    #[test]
-    fn test_as_mut() {
-        let mut my_proxied = MyProxied { val: "Hello World".to_string() };
-
-        let mut my_mut = my_proxied.as_mut();
-        my_mut.set("Hello indeed".to_string());
-
-        let val_after_set = my_mut.as_view().val().to_string();
-        assert_that!(my_proxied.val, eq(val_after_set));
-        assert_that!(my_proxied.val, eq("Hello indeed"));
-    }
-
     fn reborrow_mut_into_view<'msg>(x: Mut<'msg, MyProxied>) -> View<'msg, MyProxied> {
         // x.as_view() fails to compile with:
         //   `ERROR: attempt to return function-local borrowed content`
         x.into_view() // OK: we return the same lifetime as we got in.
     }
 
-    #[test]
+    #[gtest]
     fn test_mut_into_view() {
         let mut my_proxied = MyProxied { val: "Hello World".to_string() };
         reborrow_mut_into_view(my_proxied.as_mut());
@@ -444,7 +354,7 @@ mod tests {
 
     fn require_unified_lifetimes<'msg>(_x: Mut<'msg, MyProxied>, _y: View<'msg, MyProxied>) {}
 
-    #[test]
+    #[gtest]
     fn test_require_unified_lifetimes() {
         let mut my_proxied = MyProxied { val: "Hello1".to_string() };
         let my_mut = my_proxied.as_mut();
@@ -461,7 +371,7 @@ mod tests {
         y: &'b View<'a, T>,
     ) -> [View<'b, T>; 2]
     where
-        T: Proxied,
+        T: MutProxied,
         'a: 'b,
     {
         // `[x, y]` fails to compile because `'a` is not the same as `'b` and the `View`
@@ -469,7 +379,7 @@ mod tests {
         [x.as_view(), y.as_view()]
     }
 
-    #[test]
+    #[gtest]
     fn test_reborrow_generic_as_view() {
         let mut my_proxied = MyProxied { val: "Hello1".to_string() };
         let mut my_mut = my_proxied.as_mut();
@@ -496,7 +406,7 @@ mod tests {
         [x.into_view(), y]
     }
 
-    #[test]
+    #[gtest]
     fn test_reborrow_generic_into_view() {
         let my_proxied = MyProxied { val: "Hello1".to_string() };
         let my_view = my_proxied.as_view();
@@ -510,13 +420,13 @@ mod tests {
 
     fn reborrow_generic_mut_into_view<'a, 'b, T>(x: Mut<'a, T>, y: View<'b, T>) -> [View<'b, T>; 2]
     where
-        T: Proxied,
+        T: MutProxied,
         'a: 'b,
     {
         [x.into_view(), y]
     }
 
-    #[test]
+    #[gtest]
     fn test_reborrow_generic_mut_into_view() {
         let mut my_proxied = MyProxied { val: "Hello1".to_string() };
         let my_mut = my_proxied.as_mut();
@@ -530,16 +440,17 @@ mod tests {
 
     fn reborrow_generic_mut_into_mut<'a, 'b, T>(x: Mut<'a, T>, y: Mut<'b, T>) -> [Mut<'b, T>; 2]
     where
-        T: Proxied,
+        T: MutProxied,
         'a: 'b,
     {
         // `[x, y]` fails to compile because `'a` is not the same as `'b` and the `Mut`
         // lifetime parameter is (conservatively) invariant.
         // `[x.as_mut(), y]` fails because that borrow cannot outlive `'b`.
-        [x.into_mut(), y]
+        let tmp: Mut<'b, T> = x.into_mut();
+        [tmp, y]
     }
 
-    #[test]
+    #[gtest]
     fn test_reborrow_generic_mut_into_mut() {
         let mut my_proxied = MyProxied { val: "Hello1".to_string() };
         let my_mut = my_proxied.as_mut();
@@ -553,17 +464,4 @@ mod tests {
             reborrow_generic_mut_into_mut::<MyProxied>(my_mut, other_mut);
         }
     }
-
-    #[test]
-    fn test_set() {
-        let mut my_proxied = MyProxied::default();
-        my_proxied.as_mut().set("hello");
-        assert_that!(my_proxied.as_view().val(), eq("hello"));
-
-        my_proxied.as_mut().set(String::from("hello2"));
-        assert_that!(my_proxied.as_view().val(), eq("hello2"));
-
-        my_proxied.as_mut().set(Cow::Borrowed("hello3"));
-        assert_that!(my_proxied.as_view().val(), eq("hello3"));
-    }
 }
