@@ -7,10 +7,15 @@
 
 //! Tests covering accessors for singular bool, int32, int64, and bytes fields.
 
+#[cfg(not(bzl))]
+mod protos;
+#[cfg(not(bzl))]
+use protos::*;
+
 use googletest::prelude::*;
 use protobuf::prelude::*;
 
-use protobuf::{Optional, ProtoBytes, ProtoStr, ProtoString};
+use protobuf::{message_eq, Optional, ProtoBytes, ProtoStr, ProtoString};
 use std::borrow::Cow;
 use std::ffi::OsString;
 use std::rc::Rc;
@@ -945,3 +950,22 @@ fn test_submsg_clear() {
     // ...but it does clear the submsg's value:
     assert_that!(m.optional_nested_message().bb(), eq(0));
 }
+
+#[gtest]
+fn test_message_eq() {
+    let mut m = TestAllTypes::new();
+    let mut sub = m.optional_nested_message_mut();
+    sub.set_bb(7);
+
+    assert_that!(message_eq(&m, &m), eq(true));
+
+    let mut m2 = TestAllTypes::new();
+    assert_that!(message_eq(&m, &m2), eq(false));
+
+    let _ = m2.optional_nested_message_mut();
+    assert_that!(message_eq(&m, &m2), eq(false));
+
+    let mut sub2 = m2.optional_nested_message_mut();
+    sub2.set_bb(7);
+    assert_that!(message_eq(&m, &m2), eq(true));
+}
