@@ -26,6 +26,7 @@
 #include "absl/container/flat_hash_map.h"
 #include "absl/container/flat_hash_set.h"
 #include "absl/strings/string_view.h"
+#include "google/protobuf/compiler/code_generator_lite.h"
 #include "google/protobuf/compiler/plugin.pb.h"
 #include "google/protobuf/descriptor.pb.h"
 #include "google/protobuf/descriptor_database.h"
@@ -57,6 +58,7 @@ struct TransitiveDependencyOptions {
   bool include_json_name = false;
   bool include_source_code_info = false;
   bool retain_options = false;
+  bool skip_dependencies = false;
 };
 
 // This class implements the command-line interface to the protocol compiler.
