@@ -25,6 +25,9 @@
 #include "google/protobuf/descriptor.pb.h"
 #include "google/protobuf/io/printer.h"
 
+// Must be included last.
+#include "google/protobuf/port_def.inc"
+
 namespace google {
 namespace protobuf {
 namespace compiler {
@@ -41,9 +44,13 @@ std::vector<Sub> Vars(const FieldDescriptor* field, const Options& opts) {
   return {
       {"Enum", enum_name},
       {"kDefault", Int32ToString(default_value->number())},
-      Sub("assert_valid",
-          is_open ? ""
-                  : absl::Substitute("assert($0_IsValid(value));", enum_name))
+      Sub("assert_valid", is_open ? ""
+                                  : absl::Substitute(
+                                        R"cc(
+                                          assert(::$0::internal::ValidateEnum(
+                                              value, $1_internal_data_));
+                                        )cc",
+                                        ProtobufNamespace(opts), enum_name))
           .WithSuffix(";"),
 
       {"cached_size_name", MakeVarintCachedSizeName(field)},
@@ -53,9 +60,8 @@ std::vector<Sub> Vars(const FieldDescriptor* field, const Options& opts) {
 
 class SingularEnum : public FieldGeneratorBase {
  public:
-  SingularEnum(const FieldDescriptor* field, const Options& opts,
-               MessageSCCAnalyzer* scc)
-      : FieldGeneratorBase(field, opts, scc), field_(field), opts_(&opts) {}
+  SingularEnum(const FieldDescriptor* field, const Options& opts)
+      : FieldGeneratorBase(field, opts), opts_(&opts) {}
   ~SingularEnum() override = default;
 
   std::vector<Sub> MakeVars() const override { return Vars(field_, *opts_); }
@@ -86,13 +92,6 @@ class SingularEnum : public FieldGeneratorBase {
     )cc");
   }
 
-  void GenerateConstructorCode(io::Printer* p) const override {
-    if (!is_oneof()) return;
-    p->Emit(R"cc(
-      $ns$::_$Msg$_default_instance_.$field_$ = $kDefault$;
-    )cc");
-  }
-
   void GenerateCopyConstructorCode(io::Printer* p) const override {
     p->Emit(R"cc(
       _this->$field_$ = from.$field_$;
@@ -103,17 +102,18 @@ class SingularEnum : public FieldGeneratorBase {
     p->Emit(R"cc(
       target = stream->EnsureSpace(target);
       target = ::_pbi::WireFormatLite::WriteEnumToArray(
-          $number$, this->_internal_$name$(), target);
+          $number$, this_._internal_$name$(), target);
     )cc");
   }
 
   void GenerateByteSize(io::Printer* p) const override {
     p->Emit(R"cc(
       total_size += $kTagBytes$ +
-                    ::_pbi::WireFormatLite::EnumSize(this->_internal_$name$());
+                    ::_pbi::WireFormatLite::EnumSize(this_._internal_$name$());
     )cc");
   }
 
+
   void GenerateConstexprAggregateInitializer(io::Printer* p) const override {
     p->Emit(R"cc(
       /*decltype($field_$)*/ $kDefault$,
@@ -142,7 +142,6 @@ class SingularEnum : public FieldGeneratorBase {
   void GenerateInlineAccessorDefinitions(io::Printer* p) const override;
 
  private:
-  const FieldDescriptor* field_;
   const Options* opts_;
 };
 
@@ -151,7 +150,7 @@ void SingularEnum::GenerateAccessorDeclarations(io::Printer* p) const {
       AnnotatedAccessors(field_, {"", "_internal_", "_internal_set_"}));
   auto vs = p->WithVars(AnnotatedAccessors(field_, {"set_"}, Semantic::kSet));
   p->Emit(R"cc(
-    $DEPRECATED$ $Enum$ $name$() const;
+    [[nodiscard]] $DEPRECATED$ $Enum$ $name$() const;
     $DEPRECATED$ void $set_name$($Enum$ value);
 
     private:
@@ -218,10 +217,8 @@ void SingularEnum::GenerateInlineAccessorDefinitions(io::Printer* p) const {
 
 class RepeatedEnum : public FieldGeneratorBase {
  public:
-  RepeatedEnum(const FieldDescriptor* field, const Options& opts,
-               MessageSCCAnalyzer* scc)
-      : FieldGeneratorBase(field, opts, scc),
-        field_(field),
+  RepeatedEnum(const FieldDescriptor* field, const Options& opts)
+      : FieldGeneratorBase(field, opts),
         opts_(&opts),
         has_cached_size_(field_->is_packed() &&
                          HasGeneratedMethods(field_->file(), opts) &&
@@ -243,7 +240,7 @@ class RepeatedEnum : public FieldGeneratorBase {
 
     if (has_cached_size_) {
       p->Emit(R"cc(
-        mutable $pbi$::CachedSize $cached_size_name$;
+        $pbi$::CachedSize $cached_size_name$;
       )cc");
     }
   }
@@ -285,7 +282,7 @@ class RepeatedEnum : public FieldGeneratorBase {
   void GenerateDestructorCode(io::Printer* p) const override {
     if (should_split()) {
       p->Emit(R"cc(
-        $field_$.DeleteIfNotDefault();
+        this_.$field_$.DeleteIfNotDefault();
       )cc");
     }
   }
@@ -302,9 +299,14 @@ class RepeatedEnum : public FieldGeneratorBase {
   }
 
   void GenerateAggregateInitializer(io::Printer* p) const override {
-    p->Emit(R"cc(
-      decltype($field_$){arena},
-    )cc");
+    p->Emit({InternalMetadataOffsetSub(p)},
+            R"cc(
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
+              decltype($field_$){$internal_metadata_offset$},
+#else
+              decltype($field_$){arena},
+#endif
+            )cc");
     if (has_cached_size_) {
       // std::atomic has no copy constructor, which prevents explicit aggregate
       // initialization pre-C++17.
@@ -327,21 +329,41 @@ class RepeatedEnum : public FieldGeneratorBase {
   }
 
   void GenerateMemberConstexprConstructor(io::Printer* p) const override {
-    p->Emit("$name$_{}");
+    p->Emit({InternalMetadataOffsetSub(p)},
+            R"cc(
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
+              $name$_{visibility, $internal_metadata_offset$}
+#else
+              $name$_ {}
+#endif
+            )cc");
     if (has_cached_size_) {
       p->Emit(",\n_$name$_cached_byte_size_{0}");
     }
   }
 
   void GenerateMemberConstructor(io::Printer* p) const override {
-    p->Emit("$name$_{visibility, arena}");
+    p->Emit({InternalMetadataOffsetSub(p)},
+            R"cc(
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
+              $name$_{visibility, $internal_metadata_offset$}
+#else
+              $name$_ { visibility, arena }
+#endif
+            )cc");
     if (has_cached_size_) {
       p->Emit(",\n_$name$_cached_byte_size_{0}");
     }
   }
 
   void GenerateMemberCopyConstructor(io::Printer* p) const override {
-    p->Emit("$name$_{visibility, arena, from.$name$_}");
+    p->Emit({InternalMetadataOffsetSub(p)},
+            R"cc(
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
+              $name$_{visibility, $internal_metadata_offset$, from.$name$_}
+#else
+              $name$_ { visibility, arena, from.$name$_ }
+#endif)cc");
     if (has_cached_size_) {
       p->Emit(",\n_$name$_cached_byte_size_{0}");
     }
@@ -361,15 +383,12 @@ class RepeatedEnum : public FieldGeneratorBase {
     }
   }
 
-  void GenerateConstructorCode(io::Printer* p) const override {}
-
   void GenerateAccessorDeclarations(io::Printer* p) const override;
   void GenerateInlineAccessorDefinitions(io::Printer* p) const override;
   void GenerateSerializeWithCachedSizesToArray(io::Printer* p) const override;
   void GenerateByteSize(io::Printer* p) const override;
 
  private:
-  const FieldDescriptor* field_;
   const Options* opts_;
   bool has_cached_size_;
 };
@@ -384,15 +403,15 @@ void RepeatedEnum::GenerateAccessorDeclarations(io::Printer* p) const {
 
   p->Emit(R"cc(
     public:
-    $DEPRECATED$ $Enum$ $name$(int index) const;
+    [[nodiscard]] $DEPRECATED$ $Enum$ $name$(int index) const;
     $DEPRECATED$ void $set_name$(int index, $Enum$ value);
     $DEPRECATED$ void $add_name$($Enum$ value);
-    $DEPRECATED$ const $pb$::RepeatedField<int>& $name$() const;
-    $DEPRECATED$ $pb$::RepeatedField<int>* $mutable_name$();
+    [[nodiscard]] $DEPRECATED$ const $pb$::RepeatedField<int>& $name$() const;
+    [[nodiscard]] $DEPRECATED$ $pb$::RepeatedField<int>* $nonnull$ $mutable_name$();
 
     private:
     const $pb$::RepeatedField<int>& $_internal_name$() const;
-    $pb$::RepeatedField<int>* $_internal_mutable_name$();
+    $pb$::RepeatedField<int>* $nonnull$ $_internal_mutable_name$();
 
     public:
   )cc");
@@ -408,6 +427,9 @@ void RepeatedEnum::GenerateInlineAccessorDefinitions(io::Printer* p) const {
     }
   )cc");
   p->Emit(R"cc(
+    //~ Note: no need to set hasbit in set_$name$(int index). Hasbits only
+    //~ need to be updated if a new element is (potentially) added, not if an
+    //~ existing element is mutated.
     inline void $Msg$::set_$name$(int index, $Enum$ value) {
       $WeakDescriptorSelfPin$;
       $assert_valid$;
@@ -421,7 +443,9 @@ void RepeatedEnum::GenerateInlineAccessorDefinitions(io::Printer* p) const {
       $WeakDescriptorSelfPin$;
       $assert_valid$;
       $TsanDetectConcurrentMutation$;
-      _internal_mutable_$name_internal$()->Add(value);
+      _internal_mutable_$name_internal$()->InternalAddWithArena(
+          internal_visibility(), GetArena(), value);
+      $set_hasbit$;
       $annotate_add$
       // @@protoc_insertion_point(field_add:$pkg.Msg.field$)
     }
@@ -436,9 +460,10 @@ void RepeatedEnum::GenerateInlineAccessorDefinitions(io::Printer* p) const {
     }
   )cc");
   p->Emit(R"cc(
-    inline $pb$::RepeatedField<int>* $Msg$::mutable_$name$()
+    inline $pb$::RepeatedField<int>* $nonnull$ $Msg$::mutable_$name$()
         ABSL_ATTRIBUTE_LIFETIME_BOUND {
       $WeakDescriptorSelfPin$;
+      $set_hasbit$;
       $annotate_mutable_list$;
       // @@protoc_insertion_point(field_mutable_list:$pkg.Msg.field$)
       $TsanDetectConcurrentMutation$;
@@ -452,12 +477,12 @@ void RepeatedEnum::GenerateInlineAccessorDefinitions(io::Printer* p) const {
         $TsanDetectConcurrentRead$;
         return *$field_$;
       }
-      inline $pb$::RepeatedField<int>* $Msg$::_internal_mutable_$name_internal$() {
+      inline $pb$::RepeatedField<int>* $nonnull$
+      $Msg$::_internal_mutable_$name_internal$() {
         $TsanDetectConcurrentRead$;
         $PrepareSplitMessageForWrite$;
         if ($field_$.IsDefault()) {
-          $field_$.Set(
-              $pb$::Arena::CreateMessage<$pb$::RepeatedField<int>>(GetArena()));
+          $field_$.Set($pb$::Arena::Create<$pb$::RepeatedField<int>>(GetArena()));
         }
         return $field_$.Get();
       }
@@ -469,7 +494,8 @@ void RepeatedEnum::GenerateInlineAccessorDefinitions(io::Printer* p) const {
         $TsanDetectConcurrentRead$;
         return $field_$;
       }
-      inline $pb$::RepeatedField<int>* $Msg$::_internal_mutable_$name_internal$() {
+      inline $pb$::RepeatedField<int>* $nonnull$
+      $Msg$::_internal_mutable_$name_internal$() {
         $TsanDetectConcurrentRead$;
         return &$field_$;
       }
@@ -486,15 +512,15 @@ void RepeatedEnum::GenerateSerializeWithCachedSizesToArray(
              [&] {
                if (has_cached_size_) {
                  p->Emit(
-                     R"cc(std::size_t byte_size = $cached_size_$.Get();)cc");
+                     R"cc(::size_t byte_size = this_.$cached_size_$.Get();)cc");
                } else {
                  p->Emit(R"cc(
-                   std::size_t byte_size = 0;
-                   auto count = static_cast<std::size_t>(this->_internal_$name$_size());
+                   ::size_t byte_size = 0;
+                   auto count = static_cast<::size_t>(this_._internal_$name$_size());
 
-                   for (std::size_t i = 0; i < count; ++i) {
+                   for (::size_t i = 0; i < count; ++i) {
                      byte_size += ::_pbi::WireFormatLite::EnumSize(
-                         this->_internal_$name$().Get(static_cast<int>(i)));
+                         this_._internal_$name$().Get(static_cast<int>(i)));
                    }
                  )cc");
                }
@@ -504,78 +530,73 @@ void RepeatedEnum::GenerateSerializeWithCachedSizesToArray(
           {
             $byte_size$;
             if (byte_size > 0) {
-              target = stream->WriteEnumPacked($number$, _internal_$name$(),
-                                               byte_size, target);
+              target = stream->WriteEnumPacked(
+                  $number$, this_._internal_$name$(), byte_size, target);
             }
           }
         )cc");
     return;
   }
   p->Emit(R"cc(
-    for (int i = 0, n = this->_internal_$name$_size(); i < n; ++i) {
+    for (int i = 0, n = this_._internal_$name$_size(); i < n; ++i) {
       target = stream->EnsureSpace(target);
       target = ::_pbi::WireFormatLite::WriteEnumToArray(
-          $number$, static_cast<$Enum$>(this->_internal_$name$().Get(i)),
+          $number$, static_cast<$Enum$>(this_._internal_$name$().Get(i)),
           target);
     }
   )cc");
 }
 
 void RepeatedEnum::GenerateByteSize(io::Printer* p) const {
+  if (has_cached_size_) {
+    ABSL_CHECK(field_->is_packed());
+    p->Emit(R"cc(
+      total_size += ::_pbi::WireFormatLite::EnumSizeWithPackedTagSize(
+          this_._internal_$name$(), $kTagBytes$, this_.$cached_size_$);
+    )cc");
+    return;
+  }
   p->Emit(
       {
-          {"add_to_size",
+          {"tag_size",
            [&] {
-             if (!field_->is_packed()) {
+             if (field_->is_packed()) {
                p->Emit(R"cc(
-                 total_size += std::size_t{$kTagBytes$} * count;
+                 data_size == 0
+                     ? 0
+                     : $kTagBytes$ + ::_pbi::WireFormatLite::Int32Size(
+                                         static_cast<::int32_t>(data_size));
                )cc");
-               return;
-             }
-
-             p->Emit(R"cc(
-               if (data_size > 0) {
-                 total_size += $kTagBytes$;
-                 total_size += ::_pbi::WireFormatLite::Int32Size(
-                     static_cast<int32_t>(data_size));
-               }
-             )cc");
-             if (has_cached_size_) {
+             } else {
                p->Emit(R"cc(
-                 $cached_size_$.Set(::_pbi::ToCachedSize(data_size));
+                 ::size_t{$kTagBytes$} *
+                     ::_pbi::FromIntSize(this_._internal_$name$_size());
                )cc");
              }
            }},
       },
       R"cc(
-        {
-          std::size_t data_size = 0;
-          auto count = static_cast<std::size_t>(this->_internal_$name$_size());
-
-          for (std::size_t i = 0; i < count; ++i) {
-            data_size += ::_pbi::WireFormatLite::EnumSize(
-                this->_internal_$name$().Get(static_cast<int>(i)));
-          }
-          total_size += data_size;
-          $add_to_size$;
-        }
+        ::size_t data_size =
+            ::_pbi::WireFormatLite::EnumSize(this_._internal_$name$());
+        ::size_t tag_size = $tag_size$;
+        total_size += data_size + tag_size;
       )cc");
 }
 }  // namespace
 
 std::unique_ptr<FieldGeneratorBase> MakeSinguarEnumGenerator(
-    const FieldDescriptor* desc, const Options& options,
-    MessageSCCAnalyzer* scc) {
-  return absl::make_unique<SingularEnum>(desc, options, scc);
+    const FieldDescriptor* desc, const Options& options) {
+  return absl::make_unique<SingularEnum>(desc, options);
 }
 
 std::unique_ptr<FieldGeneratorBase> MakeRepeatedEnumGenerator(
-    const FieldDescriptor* desc, const Options& options,
-    MessageSCCAnalyzer* scc) {
-  return absl::make_unique<RepeatedEnum>(desc, options, scc);
+    const FieldDescriptor* desc, const Options& options) {
+  return absl::make_unique<RepeatedEnum>(desc, options);
 }
 
 }  // namespace cpp
 }  // namespace compiler
 }  // namespace protobuf
 }  // namespace google
+
+#include "google/protobuf/port_undef.inc"
