@@ -112,16 +112,16 @@ bool FileGenerator::ShouldSkipDependencyImports(
 }
 
 FileGenerator::FileGenerator(const FileDescriptor* file, const Options& options)
-    : file_(file), options_(options), scc_analyzer_(options) {
+    : file_(file), options_(options) {
   std::vector<const Descriptor*> msgs = FlattenMessagesInFile(file);
   std::vector<const Descriptor*> msgs_topologically_ordered =
-      TopologicalSortMessagesInFile(file, scc_analyzer_);
+      TopologicalSortMessagesInFile(file, *options.scc_analyzer);
   ABSL_CHECK(msgs_topologically_ordered.size() == msgs.size())
       << "Size mismatch";
 
   for (size_t i = 0; i < msgs.size(); ++i) {
-    message_generators_.push_back(std::make_unique<MessageGenerator>(
-        msgs[i], variables_, i, options, &scc_analyzer_));
+    message_generators_.push_back(
+        std::make_unique<MessageGenerator>(msgs[i], variables_, i, options));
     message_generators_.back()->AddGenerators(&enum_generators_,
                                               &extension_generators_);
   }
@@ -153,8 +153,8 @@ FileGenerator::FileGenerator(const FileDescriptor* file, const Options& options)
   }
 
   for (int i = 0; i < file->extension_count(); ++i) {
-    extension_generators_.push_back(std::make_unique<ExtensionGenerator>(
-        file->extension(i), options, &scc_analyzer_));
+    extension_generators_.push_back(
+        std::make_unique<ExtensionGenerator>(file->extension(i), options));
   }
 
   for (int i = 0; i < file->weak_dependency_count(); ++i) {
@@ -185,7 +185,9 @@ void FileGenerator::GenerateFile(io::Printer* p, GeneratedFileType file_type,
     #include <type_traits>
     #include <utility>
 
+    // clang-format off
     $cb$;
+    // clang-format on
 
     #endif  // $guard$
   )");
@@ -492,12 +494,13 @@ void FileGenerator::GenerateSourceIncludes(io::Printer* p) {
 
   IncludeFile("third_party/protobuf/io/coded_stream.h", p);
   IncludeFile("third_party/protobuf/generated_message_tctable_impl.h", p);
+  IncludeFile("third_party/protobuf/internal_visibility.h", p);
   // TODO This is to include parse_context.h, we need a better way
   IncludeFile("third_party/protobuf/extension_set.h", p);
   IncludeFile("third_party/protobuf/generated_message_util.h", p);
   IncludeFile("third_party/protobuf/wire_format_lite.h", p);
 
-  if (ShouldVerify(file_, options_, &scc_analyzer_)) {
+  if (ShouldVerify(file_, options_)) {
     IncludeFile("third_party/protobuf/wire_format_verify.h", p);
   }
 
@@ -586,7 +589,7 @@ void FileGenerator::GenerateSourceDefaultInstance(int idx, io::Printer* p) {
         },
         R"cc(
           struct $type$ {
-            PROTOBUF_CONSTEXPR $type$() : _instance{$default$} {}
+            constexpr $type$() : _instance{$default$} {}
             union {
               $class$ _instance;
             };
@@ -636,7 +639,7 @@ void FileGenerator::GenerateSourceDefaultInstance(int idx, io::Printer* p) {
         },
         R"cc(
           struct $type$ {
-            PROTOBUF_CONSTEXPR $type$() : _instance(::_pbi::ConstantInitialized{}) {}
+            constexpr $type$() : _instance(::_pbi::ConstantInitialized{}) {}
             ~$type$() {}
             //~ _instance must be the first member.
             union {
@@ -659,7 +662,7 @@ void FileGenerator::GenerateSourceDefaultInstance(int idx, io::Printer* p) {
         },
         R"cc(
           struct $type$ {
-            PROTOBUF_CONSTEXPR $type$() : _instance(::_pbi::ConstantInitialized{}) {}
+            constexpr $type$() : _instance(::_pbi::ConstantInitialized{}) {}
             ~$type$() {}
             union {
               $class$ _instance;
@@ -671,27 +674,6 @@ void FileGenerator::GenerateSourceDefaultInstance(int idx, io::Printer* p) {
         )cc");
   }
 
-  for (int i = 0; i < generator->descriptor()->field_count(); ++i) {
-    const FieldDescriptor* field = generator->descriptor()->field(i);
-    if (!IsStringInlined(field, options_)) {
-      continue;
-    }
-
-    // Force the initialization of the inlined string in the default instance.
-    p->Emit(
-        {
-            {"class", ClassName(generator->descriptor())},
-            {"field", FieldName(field)},
-            {"default", DefaultInstanceName(generator->descriptor(), options_)},
-            {"member", FieldMemberName(field, ShouldSplit(field, options_))},
-        },
-        R"cc(
-          PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::std::true_type
-              $class$::Impl_::_init_inline_$field$_ =
-                  ($default$._instance.$member$.Init(), ::std::true_type{});
-        )cc");
-  }
-
   if (options_.lite_implicit_weak_fields) {
     p->Emit(
         {
@@ -736,8 +718,7 @@ void FileGenerator::GetCrossFileReferencesForField(const FieldDescriptor* field,
     return;
   }
 
-  if (IsImplicitWeakField(field, options_, &scc_analyzer_) ||
-      IsWeak(field, options_)) {
+  if (IsImplicitWeakField(field, options_) || IsWeak(field, options_)) {
     refs->weak_default_instances.insert(msg);
   }
 }
@@ -1057,6 +1038,20 @@ static void GatherAllCustomOptionTypes(
     return;
   }
 
+  const auto is_import_option = [file](const FieldDescriptor* field) {
+    if (!field->is_extension() || field->message_type() == nullptr) {
+      return false;
+    }
+
+    for (int i = 0; i < file->option_dependency_count(); ++i) {
+      if (field->file()->name() == file->option_dependency_name(i)) {
+        return true;
+      }
+    }
+
+    return false;
+  };
+
   // It's easier to inspect file as a proto, because we can use reflection on
   // the proto to iterate over all content.
   // However, we can't use the generated proto linked into the proto compiler
@@ -1085,6 +1080,11 @@ static void GatherAllCustomOptionTypes(
     reflection.ListFields(msg, &fields);
 
     for (auto* field : fields) {
+      if (is_import_option(field)) {
+        // If it is an `import option` dependency we can skip it.
+        continue;
+      }
+
       if (field->is_extension()) {
         // Always add the extended.
         const Descriptor* desc = msg.GetDescriptor();
@@ -1242,7 +1242,7 @@ void FileGenerator::GenerateReflectionInitializationCode(io::Printer* p) {
 
   FileDescriptorProto file_proto = StripSourceRetentionOptions(*file_);
   std::string file_data;
-  file_proto.SerializeToString(&file_data);
+  ABSL_CHECK(file_proto.SerializeToString(&file_data));
 
   auto desc_name = UniqueName("descriptor_table_protodef", file_, options_);
   p->Emit(
@@ -1611,11 +1611,11 @@ void FileGenerator::GenerateLibraryIncludes(io::Printer* p) {
     ABSL_CHECK(!options_.opensource_runtime);
     IncludeFile("third_party/protobuf/weak_field_map.h", p);
   }
-  if (HasLazyFields(file_, options_, &scc_analyzer_)) {
+  if (HasLazyFields(file_, options_)) {
     ABSL_CHECK(!options_.opensource_runtime);
     IncludeFile("third_party/protobuf/lazy_field.h", p);
   }
-  if (ShouldVerify(file_, options_, &scc_analyzer_)) {
+  if (ShouldVerify(file_, options_)) {
     IncludeFile("third_party/protobuf/wire_format_verify.h", p);
   }
   if (options_.experimental_use_micro_string) {
