@@ -13,6 +13,7 @@
 #include "absl/log/absl_check.h"
 #include "google/protobuf/descriptor.h"
 #include "google/protobuf/generated_message_bases.h"
+#include "google/protobuf/internal_metadata_locator.h"
 #include "google/protobuf/port.h"
 #include "google/protobuf/repeated_ptr_field.h"
 #include "google/protobuf/unittest.pb.h"
@@ -48,25 +49,46 @@ struct MockZeroFieldsBase : public MockMessageBase {
 ABSL_CHECK_MESSAGE_SIZE(MockZeroFieldsBase, 24);
 
 struct MockExtensionSet {
+#ifndef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_EXTENSION_SET
   void* arena;       // 8 bytes
+#endif
   int16_t capacity;  // 4 bytes
   int16_t size;      // 4 bytes
   void* data;        // 8 bytes
 };
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_EXTENSION_SET
+ABSL_CHECK_MESSAGE_SIZE(MockExtensionSet, 16);
+#else
 ABSL_CHECK_MESSAGE_SIZE(MockExtensionSet, 24);
+#endif
 
 struct MockRepeatedPtrField {
+#ifndef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
   void* arena;       // 8 bytes
+#endif
   int current_size;  // 4 bytes
   int total_size;    // 4 bytes
   void* data;        // 8 bytes
 };
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
+ABSL_CHECK_MESSAGE_SIZE(MockRepeatedPtrField, 16);
+#else
 ABSL_CHECK_MESSAGE_SIZE(MockRepeatedPtrField, 24);
+#endif
 
 struct MockRepeatedField {
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
+  internal::InternalMetadataResolver resolver;  // 4 bytes
+  int size;                                     // 4 bytes
+  union {                                       // 8 bytes
+    void* heap_rep;
+    uint8_t soo_capacity[internal::kSooCapacityBytes];
+  };
+#else
   int current_size;  // 4 bytes
   int total_size;    // 4 bytes
   void* data;        // 8 bytes
+#endif
 };
 ABSL_CHECK_MESSAGE_SIZE(MockRepeatedField, 16);
 
@@ -98,7 +120,11 @@ TEST(GeneratedMessageTest, EmptyMessageWithExtensionsSize) {
     PROTOBUF_TSAN_DECLARE_MEMBER;                  // 0-4 bytes
     // + 0-4 bytes of padding
   };
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_EXTENSION_SET
+  ABSL_CHECK_MESSAGE_SIZE(MockGenerated, 40);
+#else
   ABSL_CHECK_MESSAGE_SIZE(MockGenerated, 48);
+#endif
   EXPECT_EQ(sizeof(proto2_unittest::TestEmptyMessageWithExtensions),
             sizeof(MockGenerated));
 }
@@ -156,13 +182,12 @@ TEST(GeneratedMessageTest, OneStringSize) {
 
   struct MockGeneratedInlinedString : public MockMessageBase {  // 16 bytes
     int has_bits[1];                                            // 4 bytes
-    int donated[1];                                             // 4 bytes
     int cached_size;                                            // 4 bytes
     std::string data;              // sizeof(std::string)
     PROTOBUF_TSAN_DECLARE_MEMBER;  // 0-4 bytes
                                    // + 0-4 bytes padding
   };
-  ABSL_CHECK_MESSAGE_SIZE(MockGeneratedInlinedString, 56);
+  ABSL_CHECK_MESSAGE_SIZE(MockGeneratedInlinedString, 48);
 
   using Type = std::conditional_t<internal::ForceInlineStringInProtoc(),
                                   MockGeneratedInlinedString, MockGenerated>;
@@ -180,7 +205,11 @@ TEST(GeneratedMessageTest, MoreStringSize) {
                                                      // + 0-4 bytes padding
       MockRepeatedPtrField data;                     // 24 bytes
     };
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
+    ABSL_CHECK_MESSAGE_SIZE(MockGenerated, 40);
+#else
     ABSL_CHECK_MESSAGE_SIZE(MockGenerated, 48);
+#endif
     EXPECT_EQ(sizeof(proto2_unittest::MoreString), sizeof(MockGenerated));
   } else {
     struct MockGenerated : public MockMessageBase {  // 16 bytes
@@ -189,7 +218,11 @@ TEST(GeneratedMessageTest, MoreStringSize) {
                                                      // + 0-4 bytes padding
       MockRepeatedPtrField data;                     // 24 bytes
     };
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
+    ABSL_CHECK_MESSAGE_SIZE(MockGenerated, 40);
+#else
     ABSL_CHECK_MESSAGE_SIZE(MockGenerated, 48);
+#endif
     EXPECT_EQ(sizeof(proto2_unittest::MoreString), sizeof(MockGenerated));
   }
 #else   // !PROTOBUF_FORCE_SPLIT
@@ -298,14 +331,16 @@ TEST(GeneratedMessageTest, FieldOrderingsSize) {
     PROTOBUF_TSAN_DECLARE_MEMBER;                  // 0-4 bytes
     // + 0-4 bytes padding
   };
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_EXTENSION_SET
+  ABSL_CHECK_MESSAGE_SIZE(MockGenerated, 72);
+#else
   ABSL_CHECK_MESSAGE_SIZE(MockGenerated, 80);
+#endif
 
   struct MockGeneratedExperiments : public MockMessageBase {  // 16 bytes
     int has_bits[1];                                          // 4 bytes
-    int donated[1];                                           // 4 bytes
     int cached_size;                                          // 4 bytes
-    // + 0-4 bytes padding
-    MockExtensionSet extensions;       // 24 bytes
+    MockExtensionSet extensions;                              // 24 bytes
     std::string my_string;             // sizeof(std::string)
     void* optional_nested_message[2];  // 16 bytes (lazy)
     int64_t my_int;                    // 8 bytes
@@ -313,7 +348,11 @@ TEST(GeneratedMessageTest, FieldOrderingsSize) {
     PROTOBUF_TSAN_DECLARE_MEMBER;      // 0-4 bytes
     // + 0-4 bytes padding
   };
-  ABSL_CHECK_MESSAGE_SIZE(MockGeneratedExperiments, 112);
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_EXTENSION_SET
+  ABSL_CHECK_MESSAGE_SIZE(MockGeneratedExperiments, 96);
+#else
+  ABSL_CHECK_MESSAGE_SIZE(MockGeneratedExperiments, 104);
+#endif
 
   struct MockGeneratedSplit : public MockMessageBase {  // 16 bytes
     int has_bits[1];                                    // 4 bytes
@@ -323,7 +362,11 @@ TEST(GeneratedMessageTest, FieldOrderingsSize) {
     PROTOBUF_TSAN_DECLARE_MEMBER;                       // 0-4 bytes
     // + 0-4 bytes padding
   };
+#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_EXTENSION_SET
+  ABSL_CHECK_MESSAGE_SIZE(MockGeneratedSplit, 48);
+#else
   ABSL_CHECK_MESSAGE_SIZE(MockGeneratedSplit, 56);
+#endif
 
 #ifndef PROTOBUF_FORCE_SPLIT
   // Make sure both or none are on for this test.
@@ -343,8 +386,6 @@ TEST(GeneratedMessageTest, TestMessageSize) {
   struct MockGenerated : public MockMessageBase {  // 16 bytes
     int has_bits[1];                               // 4 bytes
     int cached_size;                               // 4 bytes
-    PROTOBUF_TSAN_DECLARE_MEMBER;                  // 0-4 bytes
-                                                   // + 0-4 bytes padding
     void* m4;                                      // 8 bytes
     int64_t m2;                                    // 8 bytes
     bool m1;                                       // 1 bytes
@@ -352,12 +393,13 @@ TEST(GeneratedMessageTest, TestMessageSize) {
                                                    // + 2 bytes padding
     int m5;                                        // 4 bytes
     int64_t m6;                                    // 8 bytes
+    PROTOBUF_TSAN_DECLARE_MEMBER;                  // 0-4 bytes
+                                                   // + 0-4 bytes padding
   };
   ABSL_CHECK_MESSAGE_SIZE(MockGenerated, 56);
 
   struct MockGeneratedInlinedString : public MockMessageBase {  // 16 bytes
     int has_bits[1];                                            // 4 bytes
-    int donated[1];                                             // 4 bytes
     int cached_size;                                            // 4 bytes
     std::string m4;                // sizeof(std::string)
     int64_t m2;                    // 8 bytes
@@ -369,7 +411,7 @@ TEST(GeneratedMessageTest, TestMessageSize) {
     PROTOBUF_TSAN_DECLARE_MEMBER;  // 0-4 bytes
                                    // + 0-4 bytes padding
   };
-  ABSL_CHECK_MESSAGE_SIZE(MockGenerated, 56);
+  ABSL_CHECK_MESSAGE_SIZE(MockGeneratedInlinedString, 72);
 
   struct MockGeneratedSplit : public MockMessageBase {  // 16 bytes
     int has_bits[1];                                    // 4 bytes
