@@ -246,8 +246,8 @@ void MessageGenerator::AddExtensionGenerators(
     const FieldDescriptor* extension = descriptor_->extension(i);
     if (!generation_options_.strip_custom_options ||
         !ExtensionIsCustomOption(extension)) {
-      extension_generators->push_back(std::make_unique<ExtensionGenerator>(
-          class_name_, extension, generation_options_));
+      extension_generators->push_back(
+          std::make_unique<ExtensionGenerator>(extension, generation_options_));
       extension_generators_.push_back(extension_generators->back().get());
     }
   }
@@ -385,20 +385,37 @@ void MessageGenerator::GenerateMessageHeader(io::Printer* printer) const {
   }
 
   if (!extension_generators_.empty()) {
-    printer->Emit({{"extension_info",
-                    [&] {
-                      for (const auto* generator : extension_generators_) {
-                        generator->GenerateMembersHeader(printer);
-                      }
-                    }}},
-                  R"objc(
+    if (generation_options_.EmitCFunctionExtensions()) {
+      printer->Emit(R"objc(
+#pragma mark - $classname$ Extensions
+
+)objc");
+      for (const auto* generator : extension_generators_) {
+        generator->GenerateFunctionsHeader(printer);
+      }
+      printer->Emit("\n");
+    }
+    if (generation_options_.EmitClassBasedExtensions()) {
+      GenerateClassBasedExtensionHeader(printer);
+    }
+  }
+}
+
+void MessageGenerator::GenerateClassBasedExtensionHeader(
+    io::Printer* printer) const {
+  printer->Emit({{"extension_methods",
+                  [&] {
+                    for (const auto* generator : extension_generators_) {
+                      generator->GenerateMethodsHeader(printer);
+                    }
+                  }}},
+                R"objc(
                     @interface $classname$ (DynamicMethods)
 
-                    $extension_info$
+                    $extension_methods$
                     @end
                   )objc");
-    printer->Emit("\n");
-  }
+  printer->Emit("\n");
 }
 
 void MessageGenerator::GenerateSource(io::Printer* printer) const {
@@ -479,7 +496,8 @@ void MessageGenerator::GenerateSource(io::Printer* printer) const {
           // If the message scopes extensions, trigger the root class
           // +initialize/+extensionRegistry as that is where the
           // runtime support for extensions lives.
-          if (!extension_generators_.empty()) {
+          if (!extension_generators_.empty() &&
+              generation_options_.EmitClassBasedExtensions()) {
             printer->Emit(R"objc(
               // Start up the root class to support the scoped extensions.
               __unused Class rootStartup = [$root_class_name$ class];
