@@ -15,6 +15,7 @@
 #include "absl/strings/str_format.h"
 #include "absl/synchronization/barrier.h"
 #include "absl/synchronization/blocking_counter.h"
+#include "absl/types/optional.h"
 #include "google/protobuf/arena.h"
 #include "google/protobuf/arena_test_util.h"
 #include "google/protobuf/map.h"
@@ -44,16 +45,12 @@ struct MapFieldTestPeer {
 };
 
 using TestMapField = ::google::protobuf::internal::MapField<
-    proto2_unittest::TestMap_MapInt32Int32Entry_DoNotUse, ::int32_t, ::int32_t,
-    ::google::protobuf::internal::WireFormatLite::TYPE_INT32,
-    ::google::protobuf::internal::WireFormatLite::TYPE_INT32>;
+    proto2_unittest::TestMap_MapInt32Int32Entry_DoNotUse, ::int32_t, ::int32_t>;
 
 class MapFieldBasePrimitiveTest : public testing::TestWithParam<bool> {
  protected:
   typedef proto2_unittest::TestMap_MapInt32Int32Entry_DoNotUse EntryType;
-  typedef MapField<EntryType, int32_t, int32_t, WireFormatLite::TYPE_INT32,
-                   WireFormatLite::TYPE_INT32>
-      MapFieldType;
+  typedef MapField<EntryType, int32_t, int32_t> MapFieldType;
 
   MapFieldBasePrimitiveTest()
       : arena_(GetParam() ? new Arena() : nullptr),
@@ -139,7 +136,7 @@ TEST_P(MapFieldBasePrimitiveTest, Arena) {
     (*map_field->MutableMap())[100] = 101;
 
     // Trigger conversion to repeated field.
-    map_field->GetRepeatedField();
+    (void)map_field->GetRepeatedField();
   }
 
   {
@@ -170,9 +167,7 @@ class MapFieldStateTest
     : public testing::TestWithParam<std::tuple<State, bool>> {
  protected:
   typedef proto2_unittest::TestMap_MapInt32Int32Entry_DoNotUse EntryType;
-  typedef MapField<EntryType, int32_t, int32_t, WireFormatLite::TYPE_INT32,
-                   WireFormatLite::TYPE_INT32>
-      MapFieldType;
+  typedef MapField<EntryType, int32_t, int32_t> MapFieldType;
   MapFieldStateTest()
       : arena_(std::get<1>(GetParam()) ? new Arena() : nullptr),
         map_field_(arena_.get()),
@@ -199,7 +194,7 @@ class MapFieldStateTest
     MapFieldBase* map_field_base = map_field;
     Map<int32_t, int32_t>* map = map_field->MutableMap();
     (*map)[0] = 0;
-    map_field_base->GetRepeatedField();
+    (void)map_field_base->GetRepeatedField();
     Expect(map_field, CLEAN, 1, 1);
   }
 
@@ -212,7 +207,7 @@ class MapFieldStateTest
   void MakeRepeatedDirty(MapFieldType* map_field) {
     MakeMapDirty(map_field);
     MapFieldBase* map_field_base = map_field;
-    map_field_base->MutableRepeatedField();
+    (void)map_field_base->MutableRepeatedField();
     // We use map_ because we don't want to disturb the syncing
     map_field->map_.clear();
 
@@ -329,7 +324,8 @@ TEST_P(MapFieldStateTest, SwapClean) {
   ArenaHolder<MapFieldType> other(arena_.get());
   AddOneStillClean(other.get());
 
-  map_field_->Swap(other.get());
+  map_field_->Swap(/*arena=*/arena_.get(), other.get(),
+                   /*other_arena=*/arena_.get());
 
   Expect(map_field_.get(), CLEAN, 1, 1);
 
@@ -352,7 +348,8 @@ TEST_P(MapFieldStateTest, SwapMapDirty) {
   ArenaHolder<MapFieldType> other(arena_.get());
   MakeMapDirty(other.get());
 
-  map_field_->Swap(other.get());
+  map_field_->Swap(/*arena=*/arena_.get(), other.get(),
+                   /*other_arena=*/arena_.get());
 
   Expect(map_field_.get(), MAP_DIRTY, 1, 0);
 
@@ -375,7 +372,8 @@ TEST_P(MapFieldStateTest, SwapRepeatedDirty) {
   ArenaHolder<MapFieldType> other(arena_.get());
   MakeRepeatedDirty(other.get());
 
-  map_field_->Swap(other.get());
+  map_field_->Swap(/*arena=*/arena_.get(), other.get(),
+                   /*other_arena=*/arena_.get());
 
   Expect(map_field_.get(), REPEATED_DIRTY, 0, 1);
 
@@ -419,7 +417,7 @@ TEST_P(MapFieldStateTest, SpaceUsedExcludingSelf) {
 }
 
 TEST_P(MapFieldStateTest, GetMapField) {
-  map_field_base_->GetRepeatedField();
+  (void)map_field_base_->GetRepeatedField();
 
   if (state_ != REPEATED_DIRTY) {
     Expect(map_field_.get(), CLEAN, 1, 1);
@@ -429,7 +427,7 @@ TEST_P(MapFieldStateTest, GetMapField) {
 }
 
 TEST_P(MapFieldStateTest, MutableMapField) {
-  map_field_base_->MutableRepeatedField();
+  (void)map_field_base_->MutableRepeatedField();
 
   if (state_ != REPEATED_DIRTY) {
     Expect(map_field_.get(), REPEATED_DIRTY, 1, 1);
@@ -440,8 +438,7 @@ TEST_P(MapFieldStateTest, MutableMapField) {
 
 using MyMapField =
     MapField<proto2_unittest::TestMap_MapInt32Int32Entry_DoNotUse, int32_t,
-             int32_t, internal::WireFormatLite::TYPE_INT32,
-             internal::WireFormatLite::TYPE_INT32>;
+             int32_t>;
 
 TEST(MapFieldTest, ConstInit) {
   // This tests that `MapField` and all its base classes can be constant
