@@ -312,6 +312,7 @@ TEST(NoFieldPresenceTest, CopyTwiceDefaultStringFieldTest) {
 
   dst = src;
   dst = src;
+  (void)dst;
 }
 
 TEST(NoFieldPresenceTest, CopyTwiceAllocatedStringFieldTest) {
@@ -325,6 +326,7 @@ TEST(NoFieldPresenceTest, CopyTwiceAllocatedStringFieldTest) {
 
   dst = src;
   dst = src;
+  (void)dst;
 }
 
 TEST(NoFieldPresenceTest, CopyTwiceEmptyStringFieldTest) {
@@ -339,6 +341,7 @@ TEST(NoFieldPresenceTest, CopyTwiceEmptyStringFieldTest) {
 
   dst = src;
   dst = src;
+  (void)dst;
 }
 
 class NoFieldPresenceSwapFieldTest : public testing::Test {
@@ -880,7 +883,7 @@ TEST(NoFieldPresenceTest, ParseEmptyStringFromWire) {
                          2);
 
   TestAllTypes message;
-  message.MergeFromString(wire);
+  ABSL_CHECK(message.MergeFromString(wire));
 
   // Implicit-presence fields don't have hazzers, so we can only verify that the
   // empty bytes field is not overwritten.
@@ -903,7 +906,7 @@ TEST(MessageTest, ParseEmptyStringFromWireOverwritesExistingField) {
   //   encoded on the wire.
   absl::string_view wire("\172\x00",  // 3:LEN 0
                          2);
-  message.MergeFromString(wire);
+  ABSL_CHECK(message.MergeFromString(wire));
 
   // Implicit-presence fields don't have hazzers, so we can only verify that the
   // empty bytes field is overwritten.
@@ -926,7 +929,7 @@ TEST(MessageTest, MergeEmptyMessageFromWire) {
   TestAllTypes message;
   ASSERT_EQ(TestAllTypes::GetDescriptor()->FindFieldByNumber(19)->name(),
             "optional_foreign_message");
-  message.MergeFromString(wire);
+  ABSL_CHECK(message.MergeFromString(wire));
 
   // Message fields always have explicit presence, so serializing the message
   // will write the original bytes back out onto the wire.
@@ -950,7 +953,7 @@ TEST(MessageTest, MergeEmptyMessageFromWireDoesNotOverwiteExisting) {
   std::string original_output_data;
   EXPECT_TRUE(message.SerializeToString(&original_output_data));
 
-  message.MergeFromString(wire);
+  ABSL_CHECK(message.MergeFromString(wire));
   EXPECT_TRUE(message.has_optional_foreign_message());
   EXPECT_EQ(message.optional_foreign_message().c(), 12);
 
@@ -1209,7 +1212,7 @@ TYPED_TEST(NoFieldPresenceSerializeTest, LazyMessageFieldHasBit) {
   TypeParam& output_sink = this->GetOutputSinkRef();
   ASSERT_TRUE(TestSerialize(message, &output_sink));
   TestAllTypes message2;
-  message2.ParseFromString(this->GetOutput());
+  ABSL_CHECK(message2.ParseFromString(this->GetOutput()));
 
   EXPECT_EQ(true, message2.has_optional_lazy_message());
   EXPECT_EQ(true, r->HasField(message2, field));
